@@ -9,9 +9,7 @@
 #include <stdlib.h>
 
 #include <liboil/liboil.h>
-#ifdef HAVE_LIBART
-#include "swfdec_render_libart.h"
-#endif
+#include "swfdec_render.h"
 
 #include "swfdec_internal.h"
 
@@ -119,6 +117,14 @@ swfdec_decoder_eof (SwfdecDecoder * s)
   }
 }
 
+void
+swfdec_decoder_set_mouse(SwfdecDecoder *s, int x, int y, int button)
+{
+  s->mouse_x = x;
+  s->mouse_y = y;
+  s->mouse_button = button;
+}
+
 int
 swfdec_decoder_parse (SwfdecDecoder * s)
 {
@@ -400,10 +406,21 @@ swfdec_decoder_set_image_size (SwfdecDecoder * s, int width, int height)
 #endif
 
   swf_config_colorspace (s);
+  swfdec_render_resize (s);
 
   return SWF_OK;
 }
 
+char *
+swfdec_decoder_get_url (SwfdecDecoder * s)
+{
+  char *url = s->url;
+
+  s->url = NULL;
+
+  return url;
+}
+
 int
 swfdec_decoder_set_colorspace (SwfdecDecoder * s, int colorspace)
 {
@@ -628,6 +645,7 @@ swf_parse_header2 (SwfdecDecoder * s)
 
   s->main_sprite->sound_chunks = g_malloc0 (sizeof (gpointer) * s->n_frames);
   s->main_sprite->actions = g_malloc0 (sizeof (gpointer) * s->n_frames);
+  s->main_sprite->sound_play = g_malloc0 (sizeof (gpointer) * s->n_frames);
   s->main_sprite->n_frames = s->n_frames;
 
   swf_config_colorspace (s);
@@ -652,7 +670,7 @@ struct tag_func_struct tag_funcs[] = {
   [ST_REMOVEOBJECT] = {"RemoveObject", swfdec_spriteseg_remove_object, 0},
 //      [ ST_DEFINEBITS         ] = { "DefineBits",     NULL,   0 },
   [ST_DEFINEBITSJPEG] = {"DefineBitsJPEG", tag_func_define_bits_jpeg, 0},
-  [ST_DEFINEBUTTON] = {"DefineButton", NULL, 0},
+  [ST_DEFINEBUTTON] = {"DefineButton", tag_func_define_button, 0},
   [ST_JPEGTABLES] = {"JPEGTables", swfdec_image_jpegtables, 0},
   [ST_SETBACKGROUNDCOLOR] =
       {"SetBackgroundColor", tag_func_set_background_color, 0},
@@ -687,7 +705,7 @@ struct tag_func_struct tag_funcs[] = {
   [ST_SERIALNUMBER] = {"SerialNumber", NULL, 0},
   [ST_GENERATORTEXT] = {"GeneratorText", NULL, 0},
   [ST_FRAMELABEL] = {"FrameLabel", tag_func_frame_label, 0},
-  [ST_SOUNDSTREAMHEAD2] = {"SoundStreamHead2", NULL, 0},
+  [ST_SOUNDSTREAMHEAD2] = {"SoundStreamHead2", tag_func_sound_stream_head, 0},
   [ST_DEFINEMORPHSHAPE] =
       {"DefineMorphShape", NULL /* tag_define_morph_shape */ , 0},
   [ST_DEFINEFONT2] = {"DefineFont2", tag_func_define_font_2, 0},
