@@ -42,8 +42,11 @@ int swfdec_decoder_set_image_size (SwfdecDecoder * s, int width, int height);
 int swfdec_decoder_set_debug_level (SwfdecDecoder * s, int level);
 
 void *swfdec_decoder_get_sound_chunk (SwfdecDecoder * s, int *length);
+char *swfdec_decoder_get_url (SwfdecDecoder * s);
 
 void swfdec_decoder_eof (SwfdecDecoder * s);
 
+void swfdec_decoder_set_mouse(SwfdecDecoder *s, int x, int y, int button);
+
 G_END_DECLS
 #endif
