@@ -106,6 +106,21 @@ struct _SwfdecDecoder
 
   /* should be in SwfdecRender */
   gboolean stopped;
+
+  void *backend_private;
+
+  char *kept_buffer;
+  GList *kept_list;
+  int kept_layers;
+
+  SwfdecActionContext *context;
+
+  int mouse_x;
+  int mouse_y;
+  int mouse_button;
+  int old_mouse_button;
+
+  char *url;
 };
 
 SwfdecDecoder *swf_init (void);
