@@ -2,25 +2,28 @@
 #ifndef _SWFDEC_INTERNAL_H_
 #define _SWFDEC_INTERNAL_H_
 
-#include "art.h"
-#include "bits.h"
 #include "color.h"
-#include "get.h"
-#include "internal.h"
 #include "proto.h"
 #include "swfdec.h"
+#include "swfdec_actions.h"
+#include "swfdec_bits.h"
+#include "swfdec_buffer.h"
 #include "swfdec_button.h"
+#include "swfdec_debug.h"
 #include "swfdec_decoder.h"
 #include "swfdec_display.h"
+#include "swfdec_font.h"
 #include "swfdec_image.h"
 #include "swfdec_layer.h"
 #include "swfdec_object.h"
+#include "swfdec_rect.h"
+#include "swfdec_render.h"
 #include "swfdec_shape.h"
 #include "swfdec_sound.h"
 #include "swfdec_sprite.h"
 #include "swfdec_text.h"
+#include "swfdec_transform.h"
 #include "swfdec_types.h"
 #include "tags.h"
 
 #endif
-
