@@ -17,7 +17,8 @@
 #include <SDL.h>
 #include "spp.h"
 
-#define DEBUG(...) fprintf(stderr,__VA_ARGS__)
+//#define DEBUG(...) fprintf(stderr,__VA_ARGS__)
+#define DEBUG(...)
 
 typedef struct _Packet Packet;
 struct _Packet
@@ -28,6 +29,7 @@ struct _Packet
 };
 static Packet *packet_get (int fd);
 static void packet_free (Packet * packet);
+static void packet_write (int fd, int code, int len, const char *s);
 static gboolean render_idle_audio (gpointer data);
 static gboolean render_idle_noaudio (gpointer data);
 static void do_help (void);
@@ -47,11 +49,11 @@ static int width;
 static int height;
 
 static gboolean enable_sound = TRUE;
+static gboolean noskip = FALSE;
 static gboolean slow = FALSE;
 static gboolean safe = FALSE;
 static gboolean hidden = FALSE;
-static gboolean embedded = FALSE;
-static unsigned long xid;
+//static unsigned long xid;
 
 static double rate;
 static int interval;
@@ -81,6 +83,7 @@ main (int argc, char *argv[])
   int starting = FALSE;
   Packet *packet;
   int fd = 0;
+  int send_fd = 1;
 
   DEBUG ("swf_play: starting player\n");
 
@@ -94,9 +97,8 @@ main (int argc, char *argv[])
         enable_sound = FALSE;
         break;
       case 'x':
-        setenv ("SDL_WINDOWID", optarg, 1);
-        xid = strtol (optarg, NULL, 0);
-        embedded = TRUE;
+        //setenv ("SDL_WINDOWID", optarg, 1);
+        //xid = strtol (optarg, NULL, 0);
         break;
       case 'p':
         standalone = FALSE;
@@ -188,10 +190,10 @@ main (int argc, char *argv[])
       }
     }
 
-    if (standalone) {
+    //if (standalone) {
       if (SDL_PollEvent (&event)) {
         did_something = TRUE;
-        //DEBUG ("%d\n", event.type);
+        DEBUG ("event %d\n", event.type);
         switch (event.type) {
           case SDL_VIDEORESIZE:
             width = event.resize.w;
@@ -199,8 +201,21 @@ main (int argc, char *argv[])
             swfdec_decoder_set_image_size (s, width, height);
             new_window ();
             break;
+          case SDL_MOUSEMOTION:
+            swfdec_decoder_set_mouse (s, event.motion.x, event.motion.y,
+                event.motion.state);
+            break;
+          case SDL_MOUSEBUTTONDOWN:
+          case SDL_MOUSEBUTTONUP:
+            swfdec_decoder_set_mouse (s, event.button.x, event.button.y,
+                event.button.state);
+            break;
           case SDL_ACTIVEEVENT:
-            DEBUG ("active: %d %d\n", event.active.gain, event.active.state);
+            if (event.active.state & SDL_APPMOUSEFOCUS) {
+              if (event.active.gain == 0) {
+                swfdec_decoder_set_mouse (s, -1, -1, 0);
+              }
+            }
             if (event.active.state & SDL_APPACTIVE) {
               hidden = !event.active.gain;
             }
@@ -209,7 +224,7 @@ main (int argc, char *argv[])
             break;
         }
       }
-    }
+    //}
 
     if (starting) {
       ret = swfdec_decoder_parse (s);
@@ -241,12 +256,20 @@ main (int argc, char *argv[])
       int now = SDL_GetTicks ();
 
       if (now >= render_time) {
+        char *url;
+
         if (enable_sound) {
           render_idle_audio (NULL);
         } else {
           render_idle_noaudio (NULL);
         }
         did_something = TRUE;
+        url = swfdec_decoder_get_url (s);
+        if (url) {
+          DEBUG ("sending URL packet\n");
+          packet_write (send_fd, SPP_GO_TO_URL, strlen(url), url);
+          g_free (url);
+        }
       }
     }
 
@@ -324,6 +347,15 @@ do_safe (int standalone)
           SDL_FillRect (sdl_screen, NULL, 0x80808000);
           SDL_UpdateRect (sdl_screen, 0, 0, width, height);
           break;
+        case SDL_MOUSEMOTION:
+          swfdec_decoder_set_mouse (s, event.motion.x, event.motion.y,
+              event.motion.state);
+          break;
+        case SDL_MOUSEBUTTONDOWN:
+        case SDL_MOUSEBUTTONUP:
+          swfdec_decoder_set_mouse (s, event.button.x, event.button.y,
+              event.button.state);
+          break;
         default:
           break;
       }
@@ -392,7 +424,11 @@ sound_setup (void)
 {
   SDL_AudioSpec wanted;
 
+  if (slow) {
+  wanted.freq = 22050;
+  }else{
   wanted.freq = 44100;
+  }
 #if G_BYTE_ORDER == 4321
   wanted.format = AUDIO_S16MSB;
 #else
@@ -450,13 +486,15 @@ render_idle_audio (gpointer data)
 
   sound_buffers = g_list_append (sound_buffers, audio_buffer);
   sound_bytes += audio_buffer->length;
+#if 0
   if (slow) {
     swfdec_buffer_ref (audio_buffer);
     sound_buffers = g_list_append (sound_buffers, audio_buffer);
     sound_bytes += audio_buffer->length;
   }
+#endif
 
-  if (sound_bytes > 20000) {
+  if (sound_bytes > 20000 || noskip) {
     video_buffer = swfdec_render_get_image (s);
   } else {
     video_buffer = NULL;
@@ -570,3 +608,18 @@ packet_free (Packet * packet)
     free (packet->data);
   free (packet);
 }
+
+static void
+packet_write (int fd, int code, int len, const char *s)
+{
+  char *buf;
+
+  buf = g_malloc (len + 8);
+  memcpy (buf, &code, 4);
+  memcpy (buf + 4, &len, 4);
+  memcpy (buf + 8, s, len);
+  write (fd, buf, len + 8);
+
+  g_free (buf);
+}
+
